
#include "rational.h"

namespace m1 {

static int pgcd(int a, int b)
{
	if (a < 0)
		a = -a;

	if (b < 0)
		b = -b;

	if (a < b)
	{
		int tmp;

		tmp = a;
		a = b;
		b = tmp;
	}

	int r = 1;
	while (r != 0)
	{
		r = a % b;
		a = b;
		b = r;
	}

	return a;
}

Rat::Rat(int num, int den)
{
	if (den == 0)
		throw "denominateur nul";

	if (num == 0)
	{
		num_ = 0;
		den_ = 1;
	}
	else
	{
		if (den < 0)
		{
			num = -num;
			den = -den;
		}

		int d = pgcd(num, den);
		num_ = num / d;
		den_ = den / d;
	}
}

std::ostream & operator<<(std::ostream & st, const Rat & r)
{
	st << r.get_num();
	if (r.get_den() != 1)
		st << '/' << r.get_den();

	return st;
}

Rat operator+(const Rat & r1, const Rat & r2)
{
	return Rat(r1.get_num() * r2.get_den() + r1.get_den() * r2.get_num(), r1.get_den() * r2.get_den());
}

Rat operator-(const Rat & r1, const Rat & r2)
{
	return Rat(r1.get_num() * r2.get_den() - r1.get_den() * r2.get_num(), r1.get_den() * r2.get_den());
}

Rat operator*(const Rat & r1, const Rat & r2)
{
	return Rat(r1.get_num() * r2.get_num(), r1.get_den() * r2.get_den());
}

Rat operator/(const Rat & r1, const Rat & r2)
{
	if (r2 == 0)
		throw "division par 0";
	
	return Rat(r1.get_num() * r2.get_den(), r1.get_den() * r2.get_num());
}

bool operator==(const Rat & r1, const Rat & r2)
{
	return (r1.get_num() * r2.get_den()) == (r1.get_den() * r2.get_num());
}

bool operator==(const Rat & r, int i)
{
	return r.get_num() == (r.get_den() * i);
}

bool operator==(int i, const Rat & r)
{
	return r == i;
}

} // namespace m1